@@ -37,6 +37,10 @@ def zk_client(host, username, password):
     return client
 
 
+def url_join(url_root, child_path):
+    return "%s/%s" % (url_root.rstrip("/"), child_path)
+
+
 class Netloc(namedtuple("Netloc", "username password host")):
     @classmethod
     def from_string(cls, netloc_string):
@@ -129,10 +133,6 @@ class Proxy(ProxyType("ProxyBase", (object,), {})):
 
     @classmethod
     def parse(cls, url_string):
-        """ default to file:// """
-        if not re.match("^\w+://", url_string):
-            url_string = "file://%s" % (url_string)
-
         return urlparse(url_string)
 
     def __enter__(self):
@@ -153,6 +153,45 @@ class Proxy(ProxyType("ProxyBase", (object,), {})):
     def children_of(self, async):
         raise NotImplementedError("children_of must be implemented")
 
+    def copy(self, dst, recursive, async, verbose):
+        # basic sanity check
+        if recursive and self.scheme == "zk" and dst.scheme == "file":
+            raise CopyError("Recursive copy from zk to fs isn't supported")
+
+        start = time.time()
+
+        src_url = self.url
+        dst_url = dst.url
+
+        with self:
+            with dst:
+                self.do_copy(dst, async, verbose)
+                if recursive:
+                    for c in self.children_of(async):
+                        self.set_url(url_join(src_url, c))
+                        dst.set_url(url_join(dst_url, c))
+                        self.do_copy(dst, async, verbose)
+
+                    # reset to base urls
+                    self.set_url(src_url)
+                    dst.set_url(dst_url)
+
+        end = time.time()
+
+        print("Copying took %.2f secs" % (round(end - start, 2)))
+
+    def do_copy(self, dst, async=False, verbose=False):
+        if verbose:
+            if async:
+                print("Copying (asynchronously) from %s to %s" % (self.url, dst.url))
+            else:
+                print("Copying from %s to %s" % (self.url, dst.url))
+
+        try:
+            dst.write_path(self.read_path())
+        except Exception as ex:
+            raise CopyError("Failed to copy: %s" % (str(ex)))
+
 
 class ZKProxy(Proxy):
     """ read/write ZooKeeper paths """
@@ -171,19 +210,25 @@ class ZKProxy(Proxy):
     def __init__(self, parse_result, exists):
         super(ZKProxy, self).__init__(parse_result, exists)
         self.client = None
+        self.need_client = True  # whether we build a client or one is provided
 
     def connect(self):
-        self.client = zk_client(self.host, self.username, self.password)
+        if self.need_client:
+            self.client = zk_client(self.host, self.username, self.password)
 
-        if self.exists is not None:
-            self.check_path()
+    def disconnect(self):
+        if self.need_client:
+            if self.client:
+                self.client.stop()
 
     def __enter__(self):
         self.connect()
 
+        if self.exists is not None:
+            self.check_path()
+
     def __exit__(self, type, value, traceback):
-        if self.client:
-            self.client.stop()
+        self.disconnect()
 
     def check_path(self):
         retval = True if self.client.exists(self.path) else False
@@ -345,60 +390,3 @@ class JSONProxy(Proxy):
         def good(k):
             return k != self.path and k.startswith(self.path)
         return list(map(lambda c: c[offs:], list(filter(good, self._tree.keys()))))
-
-
-def do_copy(src, dst, async=False, verbose=False):
-    if verbose:
-        if async:
-            print("Copying (asynchronously) from %s to %s" % (src.url, dst.url))
-        else:
-            print("Copying from %s to %s" % (src.url, dst.url))
-
-    try:
-        dst.write_path(src.read_path())
-    except Exception as ex:
-        raise CopyError("Failed to copy: %s" % (str(ex)))
-
-
-def url_join(url_root, child_path):
-    if url_root.endswith("/"):
-        return "%s%s" % (url_root, child_path)
-    else:
-        return "%s/%s" % (url_root, child_path)
-
-
-def copy(src_url, dst_url, recursive=False, overwrite=False, async=False, verbose=False):
-    """
-       src and dst can be any of:
-
-       file://<path>
-       zk://[user:passwd@]host/<path>
-       json://!some!path!backup.json/some/path
-
-       with a few restrictions (i.e.: bare in mind the semantic differences
-       that znodes have with filesystem directories - so recursive copying
-       from znodes to an fs could lose data, but to a JSON file it would
-       work just fine.
-    """
-    src = Proxy.from_string(src_url, True)
-    dst = Proxy.from_string(dst_url, None if overwrite else False)
-
-    # basic sanity check
-    if recursive and src.scheme == "zk" and dst.scheme == "file":
-        raise CopyError("Recursive copy from zk to fs isn't supported")
-
-    start = time.time()
-
-    with src:
-        with dst:
-            do_copy(src, dst, async, verbose)
-            if recursive:
-                children = src.children_of(async)
-                for c in children:
-                    src.set_url(url_join(src_url, c))
-                    dst.set_url(url_join(dst_url, c))
-                    do_copy(src, dst, async, verbose)
-
-    end = time.time()
-
-    print("Copying took %.2f secs" % (round(end - start, 2)))
